from inspect import getmembers
from statistics import geometric_mean
import bpy
from .functions import *
classes = []


class OBJECT_OT_QuickDestroyFracture(bpy.types.Operator):
    """Fracture Objects"""
    bl_idname = "object.quick_destroy_fracture"
    bl_label = "Quick Destroy Fracture"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        active_obj = context.active_object

        
        # Ensure an object is selected
        if not active_obj or not active_obj.type == 'MESH':
            self.report({'WARNING'}, "No mesh object selected")
            return {'CANCELLED'}
        active_obj['Q_des_mods'] = {}
        mods_list = []
        
        Q_group = active_obj.vertex_groups.new(name='Q_destroy')
        active_obj['Q_des_mods']['group'] = Q_group.name

        # subdivision mod
        sub_mod = active_obj.modifiers.new(name="Q_desubs", type='SUBSURF')
        sub_mod.subdivision_type = 'SIMPLE'
        sub_mod.levels = 0
        mods_list.append(sub_mod.name)
        

        # Add a decimate modifier
        decimate_mod = active_obj.modifiers.new(name="Q_deci", type='DECIMATE')
        decimate_mod.ratio = .45
        decimate_mod.vertex_group = Q_group.name
        mods_list.append(decimate_mod.name)


        # Add a particle system named 'Q_destroy'
        d_psys = active_obj.modifiers.new('Q_destroy','PARTICLE_SYSTEM')
        psys = active_obj.particle_systems[d_psys.name]  # Get the new particle system
        mods_list.append(d_psys.name)
        # Configure the particle system here

        
        psys.vertex_group_density = Q_group.name
        psys.settings.normal_factor = 0
        psys.settings.frame_end =1
        psys.settings.use_modifier_stack = 1
        psys.settings.render_type = 'OBJECT'
        psys.settings.particle_size = .9
        psys.settings.effector_weights.gravity = 0
        
        explode_mod = active_obj.modifiers.new(name="Q_explode", type='EXPLODE')
        explode_mod.use_edge_cut = True
        explode_mod.use_size = True
        mods_list.append(explode_mod.name)
        
 

        solid_mod = active_obj.modifiers.new(name="Q_solidfy", type='SOLIDIFY')
        solid_mod.thickness = .05
        solid_mod.material_offset = 1
        mods_list.append(solid_mod.name)
        
        active_obj['Q_des_mods']['mods_list'] = mods_list
        bpy.context.scene.frame_current = 1

        context.view_layer.update()

        return {'FINISHED'}
    
class OBJECT_OT_MakeLinksConnections(bpy.types.Operator):
    """Make Links & Connections"""
    bl_idname = "object.make_links_connections"
    bl_label = "Make Links & Connections"
    bl_options = {'REGISTER', 'UNDO'}

    neighbors_count: bpy.props.IntProperty(name="Neighbors Count", default=5, min=1, max=10)
    max_connections: bpy.props.IntProperty(name="Max Connections", default=10, min=1, max=20)

    def execute(self, context):
        coll = main(context, self.neighbors_count, self.max_connections)
        if coll:
            bpy.ops.object.select_all(action='DESELECT')
            for obj in coll.objects:
                obj.select_set(True)
        bpy.ops.transform.resize(value=(0.5, 0.5, 0.5))

        return {'FINISHED'}
    
class OBJECT_OT_Makeselection(bpy.types.Operator):
    """Make Selection"""
    bl_idname = "object.qd_make_selection"
    bl_label = "Make Selection"
    bl_options = {'REGISTER', 'UNDO'}

    type: bpy.props.StringProperty(name="type")

    def execute(self, context):
        if self.type == 'CONSTRAINTS':
            if context.selected_objects:
                for obj in context.selected_objects:
                    if obj.type != 'EMPTY':
                        obj.select_set(False)
                    else: 
                       if not obj.rigid_body_constraint:
                            obj.select_set(0)

        if self.type == 'WEIGHT':
            if context.selected_objects:
                if context.object.type=='MESH':
                    if context.object.mode == 'WEIGHT_PAINT':
                        bpy.ops.object.mode_set(mode='OBJECT')
                    else:
                        bpy.ops.object.mode_set(mode='WEIGHT_PAINT')
                        if get_mods():
                            group = context.object['Q_des_mods']['group']
                            group = context.object.vertex_groups.get(group)
                            if group:
                                context.object.vertex_groups.active_index = group.index
                        



        return {'FINISHED'}
    

class OBJECT_OT_removefractures(bpy.types.Operator):
    """Remove fractures"""
    bl_idname = "object.qdest_remove_fractures"
    bl_label = "Remove Fractures"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        mods=  get_mods()

        for elem in mods:
            if elem in context.object.modifiers:
                context.object.modifiers.remove(context.object.modifiers[elem])
        
        if 'Q_des_mods' in context.object.keys():
            context.object.pop('Q_des_mods')
        
        return {'FINISHED'}
    


class OBJECT_OT_make_rigid(bpy.types.Operator):
    """Remove fractures"""
    bl_idname = "object.qdest_make_rigid"
    bl_label = "Into Rigid body"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        original = context.object

        bpy.ops.object.duplicate_move(OBJECT_OT_duplicate={"linked":False,})
        dup = context.object
        dup.name  = dup.name+'_fractures_'

        bpy.ops.object.select_all(action='DESELECT')

        context.view_layer.objects.active = original
        context.object.select_set(1)
        bpy.ops.object.qdest_remove_fractures()
        bpy.context.object.display_type = 'WIRE'
        bpy.ops.object.select_all(action='DESELECT')

        context.view_layer.objects.active = dup
        context.object.select_set(1)
        bpy.ops.object.convert(target='MESH')

        if 'Q_des_mods' in context.object.keys():
            context.object.pop('Q_des_mods')

        bpy.ops.object.mode_set(mode='EDIT')    
        bpy.ops.mesh.separate(type='LOOSE')
        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
               
        new_collection = bpy.data.collections.new(context.object.name+ '_Qdestroy_fractures')
        bpy.context.scene.collection.children.link(new_collection)
        for obj in bpy.context.selected_objects:
            for coll in obj.users_collection:
                coll.objects.unlink(obj)
            
            new_collection.objects.link(obj)
        bpy.context.view_layer.update()
        return {'FINISHED'}
    

classes.append(OBJECT_OT_QuickDestroyFracture)
classes.append( OBJECT_OT_MakeLinksConnections)
classes.append( OBJECT_OT_removefractures)
classes.append( OBJECT_OT_make_rigid)
classes.append( OBJECT_OT_Makeselection)